@@ -103,13 +103,12 @@ delta_g <- function(R){
 #' @param R The number of individuals marked and released at sampling occasion g from batch group g; g = 1,2,...,G. This must be an integer.
 #' @param begin_g The beginning of the occasion.
 #' @param end_g The end of the occasion.
-#' @param cores Number of cores for parallel.
 #' @return fr returns the log sum of the Hidden Markov Model.
 #' @rdname batchLL
 #' @importFrom parallel mclapply
 
 
-batchLL <- function(phi, p, R, begin_g, end_g, cores){
+batchLL <- function(phi, p, R, begin_g, end_g){
 
 
   U <- R[1]; u <- R[-1]
@@ -117,7 +116,7 @@ batchLL <- function(phi, p, R, begin_g, end_g, cores){
   #--------------------------------------
   # Rcpp_Armadillo code embedded in R
 
-  f1 <- function(x){gamma_gt(U, x, cores = cores)}
+  f1 <- function(x){gamma_gt(U, x)}
   gm <- parallel::mclapply(phi, f1)
 
   #---------------------------------------
@@ -166,8 +165,9 @@ batchLL <- function(phi, p, R, begin_g, end_g, cores){
 #'
 #' @param par Initial values for the parameters to be optimized over.
 #' @param data A capture-recapture data matrix or data frame.
+#' @param covariate_phi This covariate placeholder for the parameter phi_t
+#' @param covariate_p This covariate placeholder for the parameter p_t
 #' @param choiceModel This chooses among different models and allows for model selection
-#' @param cores The number of cores for parallelization.
 #' @return Negative Log-likelihood value of the likelihood function
 #' @rdname batchMarkHmmLL
 #' @export
@@ -180,96 +180,182 @@ batchLL <- function(phi, p, R, begin_g, end_g, cores){
 #' library(extBatchMarking)
 #' # Initial parameter
 #' theta <- c(0, -1)
-#' res1  <- batchMarkHmmLL(par        = theta,
-#'                        data        = WeatherLoach,
-#'                        choiceModel = "model4",
-#'                        cores       = 1)
+#' res1  <- batchMarkHmmLL(par          = theta,
+#'                        data          = WeatherLoach,
+#'                        choiceModel   = "model4",
+#'                        covariate_phi = NULL,
+#'                        covariate_p   = NULL)
 #' res1
 #'
 
 
-
-batchMarkHmmLL <- function(par, data, choiceModel = c("model1", "model2", "model3", "model4"), cores){
+batchMarkHmmLL <- function(par = NULL, data, covariate_phi = NULL, covariate_p = NULL, choiceModel = c("model1", "model2", "model3", "model4")){
 
   if(!is.matrix(data)) data <- as.matrix(unname(data))
-  if(nrow(data) != ncol(data)) stop(paste0("Error: nrow must be equal to the ncol of the data"))
+  if(nrow(data) != ncol(data)) stop(paste0("nrow must be equal to the ncol of the data"))
 
   # Number of capture and recapture
   R <- data[2:nrow(data), 1:ncol(data)]
   Ringed <- R[,1]; Mobs <- R[,-1]
 
+  # ChoiceModel must be one of the list
+  if(!any(choiceModel == c("model1", "model2", "model3", "model4")))
+    stop("choiceModel must be one of the following: 'model1', 'model2', 'model3', or 'model4'")
+
+  # Check if length of par is appropriate for each model
+  if(is.null(par) & choiceModel == "model1") {par <- c(rep(0, ncol(data)-1), rep(-1, ncol(data)-1))}
+  else if(!is.null(par) & choiceModel == "model1"){if(length(par) != 2*(ncol(data)-1)) stop(paste0("par must be length: ", 2*(ncol(data)-1), ", not: ", length(par)))}
+  if(is.null(par) & choiceModel == "model2") {par <- c(0, rep(-1, ncol(data)))}
+  else if(!is.null(par) & choiceModel == "model2"){if(length(par) != ncol(data)) stop(paste0("par must be length: ", ncol(data), ", not: ", length(par)))}
+  if(is.null(par) & choiceModel == "model3") {par <- c(0, rep(-1, ncol(data)-1))}
+  else if(!is.null(par) & choiceModel == "model3"){if(length(par) != ncol(data)) stop(paste0("par must be length: ", ncol(data), ", not: ", length(par)))}
+  if(is.null(par) & choiceModel == "model4") {par <- c(0, -1)}
+  else if(!is.null(par) & choiceModel == "model4"){if(length(par) != 2) stop(paste0("par must be length: ", 2, ", not: ", length(par)))}
+
+
   if(choiceModel == "model1"){
 
     # (phi_t, p_t)
 
-    Xphi <- diag(nrow(R))
-    Xp   <- diag(nrow(R))
+    if(is.null(covariate_phi) & is.null(covariate_p)){
 
-    # Parameters
-    betaphi <- par[1:ncol(Xphi)]
-    betap   <- par[(ncol(Xphi)+1):(ncol(Xp)+ncol(Xphi))]
-    phi     <- batchLogit(t(Xphi) %*% betaphi)
-    p       <- batchLogit(t(Xp) %*% betap)
+      Xphi <- diag(nrow(R))
+      Xp   <- diag(nrow(R))
+
+      # Parameters
+      betaphi <- par[1:ncol(Xphi)]
+      betap   <- par[(ncol(Xphi)+1):(ncol(Xp)+ncol(Xphi))]
+      phi     <- batchLogit(t(Xphi) %*% betaphi)
+      p       <- batchLogit(t(Xp) %*% betap)
+
+    }else if(is.null(covariate_phi) & !is.null(covariate_p)){
+
+      Xphi      <- diag(nrow(R))
+      betaphi   <- par[1:ncol(Xphi)]
+      phi       <- batchLogit(t(Xphi) %*% betaphi)
+      covariate <- scale(covariate_p)
+      Xp        <- matrix(c(rep(1, nrow(R)), covariate), nrow = nrow(R))
+      betap     <- par[(ncol(Xphi)+1):(nrow(Xp)+ncol(Xphi))]
+      p         <- batchLogit(as.matrix(rowSums(Xp * betap)))
+
+    }else if(!is.null(covariate_phi) & is.null(covariate_p)){
+
+      Xp        <- diag(nrow(R))
+      covariate <- scale(covariate_phi)
+      Xphi      <- matrix(c(rep(1, nrow(R)), covariate), nrow = nrow(R))
+      betap     <- par[(ncol(Xphi)+1):(ncol(Xp)+ncol(Xphi))]
+      p         <- batchLogit(t(Xp) %*% betap)
+      betaphi   <- par[(ncol(Xp)+1):(nrow(Xphi)+ncol(Xp))]
+      phi       <- batchLogit(as.matrix(rowSums(Xphi * betaphi)))
+
+    }else{
+
+      covariate1 <- scale(covariate_p)
+      covariate2 <- scale(covariate_phi)
+      Xphi      <- matrix(c(rep(1, nrow(R)), covariate2), nrow = nrow(R))
+      Xp        <- matrix(c(rep(1, nrow(R)), covariate1), nrow = nrow(R))
+      betap     <- par[(ncol(Xphi)+1):(nrow(Xp)+ncol(Xphi))]
+      p         <- batchLogit(as.matrix(rowSums(Xp * betap)))
+      betaphi   <- par[(ncol(Xp)+1):(nrow(Xphi)+ncol(Xp))]
+      phi       <- batchLogit(as.matrix(rowSums(Xphi * betaphi)))
+
+    }
 
   }
 
   else if(choiceModel == "model2"){
 
     # (phi_t, p_1) is model 2
-
-    Xp   <- matrix(1, ncol = 1, nrow = nrow(R))
-    Xphi <- diag(nrow(R))
-
     # Change the dimension of the Parameters
-    betap   <- par[ncol(Xp)]
-    betaphi <- par[(ncol(Xp)+1):(ncol(Xphi)+ncol(Xp))]
-    p       <- batchLogit(Xp %*% t(betap))
-    phi     <- batchLogit(t(Xphi) %*% betaphi)
+
+    Xp     <- matrix(1, ncol = 1, nrow = nrow(R))
+    betap  <- par[ncol(Xp)]
+    p      <- batchLogit(Xp %*% t(betap))
+
+    if(is.null(covariate_phi) & is.null(covariate_p)){
+
+      Xphi    <- diag(nrow(R))
+      betaphi <- par[(ncol(Xp)+1):(ncol(Xphi)+ncol(Xp))]
+      phi     <- batchLogit(t(Xphi) %*% betaphi)
+
+    }else if(is.null(covariate_phi) & !is.null(covariate_p)){
+      stop(paste0("Only supply covariate_phi for model 2. Hint: Set covariate_p = NULL"))
+    }else if(!is.null(covariate_phi) & !is.null(covariate_p)){
+      stop(paste0("You cannot supply covariate_p for 1-dimensional p in model 2."))
+    }else{
+
+      covariate <- scale(covariate_phi)
+      Xphi      <- matrix(c(rep(1, nrow(R)), covariate), nrow = nrow(R))
+      betaphi   <- par[(ncol(Xp)+1):(nrow(Xphi)+ncol(Xp))]
+      phi       <- batchLogit(as.matrix(rowSums(Xphi * betaphi)))
+
+    }
 
   }
 
   else if(choiceModel == 'model3'){
 
     # (phi_1, p_t) is model 3
-    Xphi <- matrix(1, ncol = 1, nrow = nrow(R))
-    Xp   <- diag(nrow(R))
-
     # Parameters
-    betaphi <- par[ncol(Xphi)]
-    betap   <- par[(ncol(Xphi)+1):(ncol(Xp)+ncol(Xphi))]
-    phi     <- batchLogit(Xphi %*% t(betaphi))
-    p       <- batchLogit(t(Xp) %*% betap)
+
+    Xphi        <- matrix(1, ncol = 1, nrow = nrow(R))
+    betaphi     <- par[ncol(Xphi)]
+    phi         <- batchLogit(Xphi %*% t(betaphi))
+
+    if(is.null(covariate_phi) & is.null(covariate_p)) {
+      Xp        <- diag(nrow(R))
+      betap     <- par[(ncol(Xphi)+1):(ncol(Xp)+ncol(Xphi))]
+      p         <- batchLogit(t(Xp) %*% betap)
+    }else if(!is.null(covariate_phi) & is.null(covariate_p)){
+      stop(paste0("Only supply covariate_p for model 3. Hint: Set covariate_phi = NULL"))
+    }else if(!is.null(covariate_phi) & !is.null(covariate_p)){
+      stop(paste0("You cannot supply covariate_phi for 1-dimensional phi in model 3."))
+    }else{
+      covariate <- scale(covariate_p)
+      Xp        <- matrix(c(rep(1, nrow(R)), covariate), nrow = nrow(R))
+      betap     <- par[(ncol(Xphi)+1):(nrow(Xp)+ncol(Xphi))]
+      p         <- batchLogit(as.matrix(rowSums(Xp * betap)))
+    }
 
   }else{
 
-    # (phi_1, p_1) is model 4
-    Xphi <- matrix(1, ncol = 1, nrow = nrow(R))
-    Xp   <- matrix(1, ncol = 1, nrow = nrow(R))
+    if(!is.null(covariate_phi) & is.null(covariate_p)){
+      stop(paste0("You cannot supply covariate_phi for 1-dimwnsional phi in model 4. Hint: Set covariate_phi = NULL"))
+    }else if(is.null(covariate_phi) & !is.null(covariate_p)){
+      stop(paste0("You cannot supply covariate_p for 1-dimwnsional p in model 4. Hint: Set covariate_p = NULL"))
+    }else if(!is.null(covariate_phi) & !is.null(covariate_p)){
+      stop(paste0("You cannot supply covariate_phi and covariate_p for 1-dimensional phi and p repspectively in model 4.
+                  Hint: Set covariate_phi = NULL and covariate_p = NULL"))
+    }else{
 
-    # Parameters
-    betaphi <- par[ncol(Xphi)]
-    betap   <- par[ncol(Xphi)+ncol(Xp)]
-    phi     <- batchLogit(Xphi %*% t(betaphi))
-    p       <- batchLogit(Xp %*% t(betap))
+      # (phi_1, p_1) is model 4
+      Xphi <- matrix(1, ncol = 1, nrow = nrow(R))
+      Xp   <- matrix(1, ncol = 1, nrow = nrow(R))
+
+      # Parameters
+      betaphi <- par[ncol(Xphi)]
+      betap   <- par[ncol(Xphi)+ncol(Xp)]
+      phi     <- batchLogit(Xphi %*% t(betaphi))
+      p       <- batchLogit(Xp %*% t(betap))
+
+    }
 
   }
 
-
   logL <- 0
 
   for (i in 1:nrow(R)) {
 
-    qw <- batchLL(phi[i:nrow(R)], p[i:nrow(R)], c(Ringed[i], Mobs[i, i:nrow(R)]), i, nrow(R),cores)
+    qw <- batchLL(phi[i:nrow(R)], p[i:nrow(R)], c(Ringed[i], Mobs[i, i:nrow(R)]), i, nrow(R))
     logL <- logL + qw
 
   }
 
-  #parallel::stopCluster(cl)
-
   return(-logL)
 }
 
 
+
 #------------------------------------------
 # Convolution of Poisson and Binomial
 #-----------------------------------------
@@ -482,9 +568,10 @@ batchUnmarkViterbi <- function(phi, p, lambda, gam, Umax, nBins, u){
 #' @param par Initial values for the parameters to be optimized over.
 #' @param data A capture-recapture data matrix or data frame.
 #' @param Umax The maximum number of the unmarked individuals in the population for capture on any occasion.
+#' @param covariate_phi This covariate placeholder for the parameter phi_t
+#' @param covariate_p This covariate placeholder for the parameter p_t
 #' @param nBins The number of bin size into which the matrix will be divided.
 #' @param choiceModel This chooses among different models and allow for model selection.
-#' @param cores The number of cores for parallelization.
 #' @return Negative Log-likelihood value of the likelihood function.
 #' @rdname batchMarkUnmarkHmmLL
 #' @export
@@ -502,34 +589,98 @@ batchUnmarkViterbi <- function(phi, p, lambda, gam, Umax, nBins, u){
 #'                              data        = WeatherLoach,
 #'                              choiceModel = "model4",
 #'                              Umax        = 1800,
-#'                              nBins       = 20,
-#'                              cores       = 1)
+#'                              nBins       = 600,
+#'                        covariate_phi = NULL,
+#'                        covariate_p   = NULL)
 #' res3
 
 
-
-batchMarkUnmarkHmmLL <- function(par, data, Umax, nBins, choiceModel = c("model1", "model2", "model3", "model4"), cores){
+batchMarkUnmarkHmmLL <- function(par, data, Umax, nBins, covariate_phi = NULL, covariate_p = NULL,
+                                 choiceModel = c("model1", "model2", "model3", "model4")){
 
   if(!is.matrix(data)) data <- as.matrix(unname(data))
-  if(nrow(data) != ncol(data)) stop(paste0("Error: nrow must be equal to the ncol of the data"))
+  if(nrow(data) != ncol(data)) stop(paste0("nrow must be equal to the ncol of the data"))
 
   # Number of capture and recapture
   R <- data[2:nrow(data), 1:ncol(data)]
   Ringed <- R[,1]; Mobs <- R[,-1]
   u <- data[1,] - c(0, colSums(Mobs))
 
+  if(!any(choiceModel == c("model1", "model2", "model3", "model4")))
+    stop("choiceModel must be one of the following: 'model1', 'model2', 'model3', or 'model4'")
+
+  if(is.null(par) & choiceModel == "model1") {par <- c(rep(0.1, ncol(data)+ncol(data)-1), 7, -1.5)}
+  else if(!is.null(par) & choiceModel == "model1"){if(length(par) != ncol(data)+ncol(data)+1) stop(paste0("par must be length: ", ncol(data)+ncol(data)+1, ", not: ", length(par)))}
+  if(is.null(par) & choiceModel == "model2") {par <- c(rep(0.1, ncol(data)), 7, -1.5)}
+  else if(!is.null(par) & choiceModel == "model2"){if(length(par) != ncol(data)+2) stop(paste0("par must be length: ", ncol(data)+2, ", not: ", length(par)))}
+  if(is.null(par) & choiceModel == "model3") {par <- c(rep(0.1, ncol(data)+1), 7, -1.5)}
+  else if(!is.null(par) & choiceModel == "model3"){if(length(par) != ncol(data)+3) stop(paste0("par must be length: ", ncol(data)+3, ", not: ", length(par)))}
+  if(is.null(par) & choiceModel == "model4") {par <- c(rep(0.1, 2), 7, -1.5)}
+  else if(!is.null(par) & choiceModel == "model4"){if(length(par) != 2*2) stop(paste0("par must be length: ", 2*2, ", not: ", length(par)))}
+
   if(choiceModel == "model1"){
 
     # (phi_t, p_t) model 1
 
-    Xphi <- diag(nrow(R))
-    Xp   <- diag(nrow(R) + 1)
+    if(is.null(covariate_phi) & is.null(covariate_p)){
 
-    # Parameters
-    betaphi <- par[1:ncol(Xphi)]
-    betap   <- par[(ncol(Xphi)+1):(ncol(Xp)+ncol(Xphi))]
-    phi     <- batchLogit(t(Xphi) %*% betaphi)
-    p       <- batchLogit(t(Xp) %*% betap)
+      Xphi <- diag(nrow(R))
+      Xp   <- diag(nrow(R) + 1)
+
+      # Parameters
+      betaphi <- par[1:ncol(Xphi)]
+      betap   <- par[(ncol(Xphi)+1):(ncol(Xp)+ncol(Xphi))]
+      phi     <- batchLogit(t(Xphi) %*% betaphi)
+      p       <- batchLogit(t(Xp) %*% betap)
+
+      loglambda <- par[ncol(Xphi)+ncol(Xp)+1]; lambda <- exp(loglambda)
+      loggamma  <- par[ncol(Xphi)+ncol(Xp)+2]; gam    <- exp(loggamma)
+
+    }else if(is.null(covariate_phi) & !is.null(covariate_p)){
+
+      Xphi      <- diag(nrow(R))
+      betaphi   <- par[1:ncol(Xphi)]
+      phi       <- batchLogit(t(Xphi) %*% betaphi)
+      covariate <- scale(covariate_p)
+      if(nrow(covariate) == nrow(R)) stop("nrow of covariate for parameter 'p' must be : ", nrow(R)+1, " and not ", nrow(covariate))
+      Xp        <- matrix(c(rep(1, (nrow(R)+1)), covariate), nrow = (nrow(R)+1))
+      betap     <- par[(ncol(Xphi)+1):(nrow(Xp)+ncol(Xphi))]
+      p         <- batchLogit(as.matrix(rowSums(Xp * betap)))
+
+      loglambda <- par[ncol(Xphi)+nrow(Xp)+1]; lambda <- exp(loglambda)
+      loggamma  <- par[ncol(Xphi)+nrow(Xp)+2]; gam    <- exp(loggamma)
+
+    }else if(!is.null(covariate_phi) & is.null(covariate_p)){
+
+      Xp        <- diag(nrow(R)+1)
+      covariate <- scale(covariate_phi)
+      if(nrow(covariate) != nrow(R)) stop("covariate for parameter 'phi' must be : ", nrow(R), " and not ", nrow(covariate))
+      Xphi      <- matrix(c(rep(1, nrow(R)), covariate), nrow = nrow(R))
+      betap     <- par[(nrow(Xphi)+1):(ncol(Xp)+nrow(Xphi))]
+      p         <- batchLogit(t(Xp) %*% betap)
+      betaphi   <- betaphi <- par[1:nrow(Xphi)]
+      phi       <- batchLogit(as.matrix(rowSums(Xphi * betaphi)))
+
+      loglambda <- par[nrow(Xphi)+ncol(Xp)+1]; lambda <- exp(loglambda)
+      loggamma  <- par[nrow(Xphi)+ncol(Xp)+2]; gam    <- exp(loggamma)
+
+    }else{
+
+      covariate1 <- scale(covariate_p)
+      covariate2 <- scale(covariate_phi)
+      if(nrow(covariate2) != nrow(R) | nrow(covariate1) == nrow(R)) stop("nrow of covariate for parameter 'phi' must be : ", nrow(R), " and not ", nrow(covariate2),
+                                                                         "; nrow of covariate for parameter 'p' must be : ", nrow(R)+1, " and not ", nrow(covariate1))
+      Xphi      <- matrix(c(rep(1, nrow(R)), covariate2), nrow = nrow(R))
+      Xp        <- matrix(c(rep(1, nrow(R)+1), covariate1), nrow = nrow(R)+1)
+      betap     <- par[(nrow(Xphi)+1):(nrow(Xp)+nrow(Xphi))]
+      p         <- batchLogit(as.matrix(rowSums(Xp * betap)))
+      betaphi   <- par[1:nrow(Xphi)]
+      phi       <- batchLogit(as.matrix(rowSums(Xphi * betaphi)))
+
+      loglambda <- par[nrow(Xphi)+nrow(Xp)+1]; lambda <- exp(loglambda)
+      loggamma  <- par[nrow(Xphi)+nrow(Xp)+2]; gam    <- exp(loggamma)
+
+    }
 
   }
 
@@ -537,46 +688,109 @@ batchMarkUnmarkHmmLL <- function(par, data, Umax, nBins, choiceModel = c("model1
 
     # (phi_t, p_1) is model 2
 
-    Xphi <- diag(nrow(R))
-    Xp   <- matrix(1, ncol = 1, nrow = nrow(R) + 1)
+    if(is.null(covariate_phi) & is.null(covariate_p)){
 
-    # Change the dimension of the Parameters
-    betap   <- par[ncol(Xp)]
-    betaphi <- par[(ncol(Xp)+1):(ncol(Xphi)+ncol(Xp))]
-    p       <- batchLogit(Xp %*% t(betap))
-    phi     <- batchLogit(t(Xphi) %*% betaphi)
+      Xphi <- diag(nrow(R))
+      Xp   <- matrix(1, ncol = 1, nrow = nrow(R) + 1)
+
+      # Change the dimension of the Parameters
+      betap   <- par[ncol(Xp)]
+      betaphi <- par[(ncol(Xp)+1):(ncol(Xphi)+ncol(Xp))]
+      p       <- batchLogit(Xp %*% t(betap))
+      phi     <- batchLogit(t(Xphi) %*% betaphi)
+
+      loglambda <- par[ncol(Xphi)+ncol(Xp)+1]; lambda <- exp(loglambda)
+      loggamma  <- par[ncol(Xphi)+ncol(Xp)+2]; gam    <- exp(loggamma)
+
+    }else if(is.null(covariate_phi) & !is.null(covariate_p)){
+      stop(paste0("Only supply covariate_phi for model 2. Hint: Set covariate_p = NULL"))
+    }else if(!is.null(covariate_phi) & !is.null(covariate_p)){
+      stop(paste0("You cannot supply covariate_p for 1-dimensional p in model 2."))
+    }else{
+
+      covariate <- scale(covariate_phi)
+      if(nrow(covariate) != nrow(R)) stop("covariate for parameter 'phi' must be : ", nrow(R), " and not ", nrow(covariate))
+      Xp        <- matrix(1, ncol = 1, nrow = nrow(R) + 1)
+      Xphi      <- matrix(c(rep(1, nrow(R)), covariate), nrow = nrow(R))
+      betap     <- par[ncol(Xp)]
+      betaphi   <- par[(ncol(Xp)+1):(nrow(Xphi)+ncol(Xp))]
+      p         <- batchLogit(Xp %*% t(betap))
+      phi       <- batchLogit(as.matrix(rowSums(Xphi * betaphi)))
+
+      loglambda <- par[nrow(Xphi)+ncol(Xp)+1]; lambda <- exp(loglambda)
+      loggamma  <- par[nrow(Xphi)+ncol(Xp)+2]; gam    <- exp(loggamma)
+
+    }
 
   }
 
   else if(choiceModel == "model3"){
 
     # (phi_1, p_t) is model 3
-    Xphi <- matrix(1, ncol = 1, nrow = nrow(R))
-    Xp   <- diag(nrow(R) + 1)
 
-    # Parameters
-    betaphi <- par[ncol(Xphi)]
-    betap   <- par[(ncol(Xphi)+1):(ncol(Xp)+ncol(Xphi))]
-    phi     <- batchLogit(Xphi %*% t(betaphi))
-    p       <- batchLogit(t(Xp) %*% betap)
+    if(is.null(covariate_phi) & is.null(covariate_p)){
+
+      Xphi <- matrix(1, ncol = 1, nrow = nrow(R))
+      Xp   <- diag(nrow(R) + 1)
+
+      # Parameters
+      betaphi <- par[ncol(Xphi)]
+      betap   <- par[(ncol(Xphi)+1):(ncol(Xp)+ncol(Xphi))]
+      phi     <- batchLogit(Xphi %*% t(betaphi))
+      p       <- batchLogit(t(Xp) %*% betap)
+
+      loglambda <- par[ncol(Xphi)+ncol(Xp)+1]; lambda <- exp(loglambda)
+      loggamma  <- par[ncol(Xphi)+ncol(Xp)+2]; gam    <- exp(loggamma)
+
+    }else if(!is.null(covariate_phi) & is.null(covariate_p)){
+      stop(paste0("Only supply covariate_p for model 3. Hint: Set covariate_phi = NULL"))
+    }else if(!is.null(covariate_phi) & !is.null(covariate_p)){
+      stop(paste0("You cannot supply covariate_phi for 1-dimensional phi in model 3."))
+    }else{
+
+      Xphi    <- matrix(1, ncol = 1, nrow = nrow(R))
+      betaphi <- par[ncol(Xphi)]
+      covariate <- scale(covariate_p)
+      if(nrow(covariate) == nrow(R)) stop("nrow of covariate for parameter 'p' must be : ", nrow(R)+1, " and not ", nrow(covariate))
+      Xp      <- matrix(c(rep(1, (nrow(R)+1)), covariate), nrow = (nrow(R)+1))
+      betap   <- par[(ncol(Xphi)+1):(nrow(Xp)+ncol(Xphi))]
+      phi     <- batchLogit(Xphi %*% t(betaphi))
+      p       <- batchLogit(as.matrix(rowSums(Xp * betap)))
+
+      loglambda <- par[ncol(Xphi)+nrow(Xp)+1]; lambda <- exp(loglambda)
+      loggamma  <- par[ncol(Xphi)+nrow(Xp)+2]; gam    <- exp(loggamma)
+
+    }
+
 
   }else{
 
     # (phi_1, p_1) is model 4
 
-    Xphi <- matrix(1, ncol = 1, nrow = nrow(R))
-    Xp   <- matrix(1, ncol = 1, nrow = nrow(R) + 1)
+    if(!is.null(covariate_phi) & is.null(covariate_p)){
+      stop(paste0("You cannot supply covariate_phi for 1-dimwnsional phi in model 4. Hint: Set covariate_phi = NULL"))
+    }else if(is.null(covariate_phi) & !is.null(covariate_p)){
+      stop(paste0("You cannot supply covariate_p for 1-dimwnsional p in model 4. Hint: Set covariate_p = NULL"))
+    }else if(!is.null(covariate_phi) & !is.null(covariate_p)){
+      stop(paste0("You cannot supply covariate_phi and covariate_p for 1-dimensional phi and p repspectively in model 4.
+                  Hint: Set covariate_phi = NULL and covariate_p = NULL"))
+    }else{
 
-    # Parameters
-    betaphi <- par[ncol(Xphi)]
-    betap   <- par[ncol(Xphi)+ncol(Xp)]
-    phi     <- batchLogit(Xphi %*% t(betaphi))
-    p       <- batchLogit(Xp %*% t(betap))
+      Xphi <- matrix(1, ncol = 1, nrow = nrow(R))
+      Xp   <- matrix(1, ncol = 1, nrow = nrow(R) + 1)
+
+      # Parameters
+      betaphi <- par[ncol(Xphi)]
+      betap   <- par[ncol(Xphi)+ncol(Xp)]
+      phi     <- batchLogit(Xphi %*% t(betaphi))
+      p       <- batchLogit(Xp %*% t(betap))
+
+      loglambda <- par[ncol(Xphi)+ncol(Xp)+1]; lambda <- exp(loglambda)
+      loggamma  <- par[ncol(Xphi)+ncol(Xp)+2]; gam    <- exp(loggamma)
+    }
 
   }
 
-  loglambda <- par[ncol(Xphi)+ncol(Xp)+1]; lambda <- exp(loglambda)
-  loggamma  <- par[ncol(Xphi)+ncol(Xp)+2]; gam    <- exp(loggamma)
 
   pm <- p[2:length(p)]; pu <- p[1:length(p)]
 
@@ -586,7 +800,7 @@ batchMarkUnmarkHmmLL <- function(par, data, Umax, nBins, choiceModel = c("model1
   # Log-likelihood for the Marked
   for (i in 1:nrow(R)) {
 
-    qw <- batchLL(phi[i:nrow(R)], pm[i:nrow(R)], c(Ringed[i], Mobs[i, i:nrow(R)]), i, nrow(R), cores)
+    qw    <- batchLL(phi[i:nrow(R)], pm[i:nrow(R)], c(Ringed[i], Mobs[i, i:nrow(R)]), i, nrow(R))
     logLm <- logLm + qw
 
   }
@@ -600,6 +814,7 @@ batchMarkUnmarkHmmLL <- function(par, data, Umax, nBins, choiceModel = c("model1
 
 }
 
+
 #---------------------------------------------------
 # Viterbi Algorithm for four different models
 #---------------------------------------------------
